# R code to draw Figure 1 in "Generalized Instrumental Variable Models", Andrew Chesher and Adam Rosen, created 21st October 2016.

# R version 2.15.1 (2012-06-22) -- "Roasted Marshmallows"
# Copyright (C) 2012 The R Foundation for Statistical Computing
# ISBN 3-900051-07-0
# Platform: x86_64-apple-darwin9.8.0/x86_64 (64-bit)

# makeauctiondata(n,seed) simulates n 2 person auctions with random number seed set to seed (default NA) and
# returns an object bids which is a n*2 matrix with lowest bids in column 1 and highest bids in column 2, one row per simulated auction.

makeauctiondata = function(n,seed = NA) {
  if (!is.na(seed)) {set.seed(seed)}
  pick1 = runif(n)>0.5
  pick2 = runif(n)>0.5
  startval1 = pick1*rnorm(n)+ (1-pick1)*rnorm(n,mean = 2.5,sd=0.5) + 10
  startval2 = pick2*rnorm(n)+ (1-pick2)*rnorm(n,mean = 2.5,sd=0.5) + 10
  oval1 = pmax(startval1,startval2)
  oval2 = pmin(startval1,startval2)
  down = abs(rnorm(n))
  bidlow = pmax(rep(0,n),oval2-down)
  up = runif(n)
  bidhigh = oval2 + up*(oval1 - oval2)
  return(list(bids = cbind(bidlow,bidhigh)))  }

# biddata is lowest and hghest bids from 1,000,000 simulated auctions.

set.seed(21102016)
biddata = makeauctiondata(1000000)

# drawbounds draws Figure 1 in the paper "Generalized Instrumental Variable Models"
# Horizontal axis: distribution function of valuations evaluated at V=v1
# Vertical axis: distribution function of valuations evaluated at V=v2
# Arguments:
#           bidsdata is matrix of auction data produced by function makeauctiondata
#           npoints is number of points at which to evaluate each bound
#           parameter to allow bounding functions to be drawn to limits of the picture 

drawbounds = function(v1,v2,bidsdata,npoints = 100,eps = 1e-5) {
  # note v2>v1 is required
  n = nrow(bidsdata$bids) # number of simulated auctions
  # probabilities for HT bounds - values of the distribution function of lowest and highest bids
  p_y1gv1 = sum(bidsdata$bids[,1]>v1)/n # lowest bids
  p_y2gv1 = sum(bidsdata$bids[,2]>v1)/n # highest bids
  
  p_y1gv2 = sum(bidsdata$bids[,1]>v2)/n # lowest bids
  p_y2gv2 = sum(bidsdata$bids[,2]>v2)/n # highest bids
  
  # HT pointwise bounds at v1 and v2
  htlv1 = 1 - sqrt(p_y1gv1)
  htuv1 = min(1 - sqrt(p_y2gv1),sqrt(1 - p_y1gv1))
  
  htlv2 = 1 - sqrt(p_y1gv2)
  htuv2 = min(1 - sqrt(p_y2gv2),sqrt(1 - p_y1gv2))
  
  par(pty = 's')
  plot(c(0,1),c(0,1),type = 'n',xlab = expression(paste(A[z],"(",11.5,")",sep="")),
       ylab = expression(paste(A[z],"(",12.5,")",sep="")),bty = 'n',cex.lab = 1.5,
       cex.axis = 1.5)
  abline(h=c(0,1),v=c(0,1),col='grey')
  polygon(x=c(htlv1,htuv1,htuv1,htlv1),y=c(htlv2,htlv2,htuv2,htuv2),border = "blue",lwd=2,col="linen") # plot HT pointwise bounds
  abline(0,1,lwd = 2,col = "orange")
  
  # Bounds involving two distinct values of V
  
  # Bounds from inequality (6.8) in GIV paper with (v1,v2,v3)=(11.5,12.5,Inf), drawn solid red in Figure 1.  
  thisp = sum( bidsdata$bids[,2] >= v2 & bidsdata$bids[,1] >= v1)/n
  jprob = thisp
  thisfv1 = seq(0,1- sqrt(thisp)-eps,length = npoints)
  thisfv2 = thisfv1 + sqrt((1-thisfv1)^2 - thisp)
  lines(thisfv1,y=thisfv2,lty = 1, col = "red",lwd=2)  
  
  # Bounds from inequality (6.8) in GIV paper with (v1,v2,v3)=(11.5,11.5,12.5), drawn dashed brown in Figure 1.  
  thisp = sum(v2 >= bidsdata$bids[,2] & bidsdata$bids[,1] >= v1)/n
  kprob = thisp
  thisfv1 = seq(0, 1- sqrt(thisp)-eps,length = npoints)
  thisfv2 = 1 - sqrt((1-thisfv1)^2 - thisp)
  lines(thisfv1,y=thisfv2,lty = 2, col = "brown",lwd=2)
  
} 

# produce Figure 1.
drawbounds(11.5,12.5,biddata,npoints = 100) 
